@@ -1,13 +1,25 @@
-"""Configuration setup for Myghty, and Paste error middleware
+"""Configuration setup for templating systems and Paste error
+middleware
 
 This module supplies pylons_config which handles setting up defaults
-for Myghty, Paste errorware, and prefixing Routes if necessary.
+for templating systems, Paste errorware, and prefixing Routes if
+necessary.
 """
 import os
 import re
+import warnings
+
 from paste.deploy.converters import asbool
+
+import pylons.legacy
 import pylons.templating
 
+request_defaults = dict(charset=None, errors='strict',
+                        decode_param_names=False,
+                        language='en-us')
+response_defaults = dict(content_type='text/html',
+                         charset='utf-8', errors='strict')
+
 class Config(object):
     """Pylons configuration object
     
@@ -20,9 +32,7 @@ class Config(object):
     ``pylons_config`` under the ``g`` object. There's several useful
     attributes of the config object most people will be interested in:
     
-    ``myghty``
-        The myghty configuration dict that was used to initialize Myghty
-    ``template_options``
+    ``tmpl_options``
         Full dict of template options that any TG compatible plugin should
         be able to parse. Comes with basic config needed for Myghty, Kid,
         and Mako.
@@ -30,14 +40,8 @@ class Config(object):
         Mapper object used for Routing. Yes, it is possible to add routes
         after your application has started running.
     ``paths``
-        An array of absolute paths that were defined in the applications
+        A dict of absolute paths that were defined in the applications
         ``config/environment.py`` module.
-    ``global_conf``
-        Global configuration passed in from Paste, this corresponds to the
-        DEFAULTS section in the config file.
-    ``app_conf``
-        Application specific configuration directives, passed in via Paste
-        from the app section of the config file.
     ``environ_config``
         Dict of environ keys for where in the environ to pickup various
         objects for registering with Pylons. If these are present then
@@ -49,14 +53,36 @@ class Config(object):
         a dict indicating how to configure the template engine with keys:
         ``engine``, ``template_root``, ``template_options``, and ``alias``
     ``default_charset``
-        The default character encoding specified to the browser via the
+        Deprecated: Use the response_settings dict instead.
+        Default character encoding specified to the browser via the
         'charset' parameter of the HTTP response's Content-Type header.
     ``strict_c``
         Whether or not the ``c`` object should throw an attribute error when
         access is attempted to an attribute that doesn't exist.
+    ``request_settings``
+        A dict of Content-Type related default settings for new instances of
+        ``paste.wsgiwrappers.WSGIRequest``. May contain the values ``charset``
+        and ``errors`` and ``decode_param_names``. Overrides the Pylons default
+        values specified by the ``request_defaults`` dict.
+    ``response_settings``
+        A dict of Content-Type related default settings for new instances of
+        ``pylons.Response``. May contain the values ``content_type``,
+        ``charset`` and ``errors``. Overrides the Pylons default values
+        specified by the ``response_defaults`` dict.
+    ``global_conf``
+        Global configuration passed in from Paste, this corresponds to the
+        DEFAULTS section in the config file.
+    ``app_conf``
+        Application specific configuration directives, passed in via Paste
+        from the app section of the config file.
     """
-    def __init__(self, tmpl_options, map, paths, environ_config=None, 
-        default_charset='UTF-8', strict_c=False):
+    def __init__(self, tmpl_options=None, map=None, paths=None, 
+                 environ_config=None, default_charset=None, strict_c=False,
+                 request_settings=None, response_settings=None):
+        if paths is None:
+            paths = {}
+        if tmpl_options is None:
+            tmpl_options = {}
         if environ_config is None:
             environ_config = {}
         self.myghty = tmpl_options
@@ -64,14 +90,26 @@ class Config(object):
         self.map = map
         self.paths = paths
         self.environ_config = environ_config
-        self.default_charset = default_charset
         self.strict_c = strict_c
-        if 'output_encoding' not in tmpl_options:
-            tmpl_options['output_encoding'] = default_charset
+        self.helpers = None
+
+        self.request_settings = request_defaults.copy()
+        if request_settings:
+            self.request_settings.update(request_settings)
+        self.response_settings = response_defaults.copy()
+        if response_settings:
+            self.response_settings.update(response_settings)
+
         self.global_conf = {}
         self.app_conf = {}
         self.template_engines = []
         self._clean_tmpl_options()
+
+        if default_charset is not None:
+            warnings.warn(pylons.legacy.default_charset_warning % \
+                              dict(klass='Config', charset=default_charset),
+                          DeprecationWarning, 2)
+            self.response_settings['charset'] = default_charset
     
     def _clean_tmpl_options(self):
         """Prase the template options, set self.myghty to have myghty options
@@ -89,7 +127,8 @@ class Config(object):
                 self.template_options[k] = v
     
     def add_template_engine(self, engine, root, options=None, alias=None):
-        """Add additional template engines for configuration on Pylons WSGI init
+        """Add additional template engines for configuration on Pylons WSGI
+        init.
         
         ``engine``
             The name of the template engine
@@ -142,14 +181,15 @@ class Config(object):
             template_options=options, alias=alias)
         self.template_engines.append(config)
     
-    def init_app(self, global_conf, app_conf, package, template_engine='pylonsmyghty'):
+    def init_app(self, global_conf, app_conf, package,
+                 template_engine='pylonsmyghty'):
         """Initialize configuration for the application
         
         ``global_config``
-            Several options are expected to be set for a Pylons web application.
-            They will be loaded from the global_config which has the main Paste
-            options. If ``debug`` is not enabled as a global config option, the
-            following option *must* be set:
+            Several options are expected to be set for a Pylons web
+            application. They will be loaded from the global_config which has
+            the main Paste options. If ``debug`` is not enabled as a global
+            config option, the following option *must* be set:
             
             * error_to - The email address to send the debug error to
             
@@ -167,8 +207,8 @@ class Config(object):
         ``package``
             The name of the application package, to be stored in the app_conf.
         ``template_engine``
-            Declare the default template engine to setup. Choices are kid, genshi,
-            mako, and pylonsmyghty (the default custom Pylons plugin).
+            Declare the default template engine to setup. Choices are kid,
+            genshi, mako, and pylonsmyghty (the default custom Pylons plugin).
         """
         self.global_conf = global_conf
         self.app_conf = app_conf
@@ -181,6 +221,8 @@ class Config(object):
         if not prefix:
             prefix = global_conf.get('prefix')
         if prefix:
+            warnings.warn(pylons.legacy.prefix_warning % prefix,
+                          DeprecationWarning, 2)
             self.map.prefix = app_conf['prefix']
             self.map._created_regs = False
         
@@ -207,6 +249,7 @@ class Config(object):
         
         # Raise a complete error for the error middleware to catch
         myghty_defaults['raise_error'] = True
+        myghty_defaults['output_encoding'] = self.response_settings['charset']
         myghty_defaults['component_root'] = [{os.path.basename(path): path} \
             for path in self.paths['templates']]
                     
@@ -250,14 +293,15 @@ class Config(object):
         # Rearrange template options as default for Mako
         defaults['mako.directories'] = self.paths['templates']
         defaults['mako.filesystem_checks'] = True
-        defaults['mako.output_encoding'] = self.default_charset
+        defaults['mako.output_encoding'] = \
+            self.response_settings['charset']
         if 'cache_dir' in app_conf:
-            defaults['mako.module_directory'] = os.path.join(app_conf['cache_dir'], 
-                                                     'templates')
+            defaults['mako.module_directory'] = \
+                os.path.join(app_conf['cache_dir'], 'templates')
         
         # Setup kid defaults
         defaults['kid.assume_encoding'] = 'utf-8'
-        defaults['kid.encoding'] = self.default_charset
+        defaults['kid.encoding'] = self.response_settings['charset']
         
         # Merge template options into defaults
         defaults.update(self.template_options)
@@ -265,7 +309,8 @@ class Config(object):
 
         # Prepare our default template engine
         if template_engine == 'pylonsmyghty':
-            self.add_template_engine('pylonsmyghty', None, myghty_template_options)
+            self.add_template_engine('pylonsmyghty', None,
+                                     myghty_template_options)
         elif template_engine == 'mako':
             self.add_template_engine('mako', '')
         elif template_engine in ['genshi', 'kid']:
